# Auto-generated from actus-dictionary-taxonomy.json
# Do not edit manually. Used for mapping acronyms to long names.

ACRONYM_TO_LONGNAME = {
    "ANN": "annuity",
    "ANX": "exoticAnnuity",
    "BCS": "boundaryControlledSwitch",
    "BNDCP": "convertibleNote",
    "BNDWR": "warrant",
    "CAPFL": "capFloor",
    "CDSWP": "creditDefaultSwap",
    "CEC": "collateral",
    "CEG": "guarantee",
    "CLM": "callMoney",
    "CLNTE": "creditLinkedNote",
    "COM": "commodity",
    "CSH": "cash",
    "EXOTi": "eXOTi",
    "FUTUR": "future",
    "FXOUT": "foreignExchangeOutright",
    "LAM": "linearAmortizer",
    "LAX": "exoticLinearAmortizer",
    "MAR": "margining",
    "NAM": "negativeAmortizer",
    "NAX": "exoticNegativeAmortizer",
    "OPTNS": "option",
    "PAM": "principalAtMaturity",
    "PBN": "perpetualBonds",
    "REP": "repurchaseAgreement",
    "SCRCR": "securitizationCreditRisk",
    "SCRMR": "securitizationMarketRisk",
    "STK": "stock",
    "SWAPS": "swap",
    "SWPPV": "plainVanillaSwap",
    "TRSWP": "totalReturnSwap",
    "UMP": "undefinedMaturityProfile",
}
